from fastapp_template.module.module_template.client.any_client import BaseClient
from fastapp_template.module.module_template.service.user.usecase import user_usecase


class DirectClient(BaseClient):
    pass
